@@ -1,4 +1,4 @@
-# !/usr/bin/env python
+#!/usr/bin/env python
 
 """
 `__main__` implementation, can be run directly or with `python -m cdd`
@@ -6,14 +6,18 @@
 from argparse import ArgumentParser, Namespace
 from codecs import decode
 from collections import deque
+from itertools import filterfalse
+from operator import eq
 from os import path
 
 from cdd import __description__, __version__
 from cdd.conformance import ground_truth
+from cdd.docstring_parsers import Style
+from cdd.doctrans import doctrans
 from cdd.gen import gen
 from cdd.openapi.gen_openapi import openapi_bulk
 from cdd.openapi.gen_routes import gen_routes, upsert_routes
-from cdd.pure_utils import pluralise
+from cdd.pure_utils import pluralise, rpartial
 from cdd.sync_properties import sync_properties
 
 
@@ -291,6 +295,44 @@ def _build_parser():
         required=True,
     )
 
+    ############
+    # doctrans #
+    ############
+    doctrans_parser = subparsers.add_parser(
+        "doctrans",
+        help=(
+            "Convert docstring format of all classes and functions within target file"
+        ),
+    )
+
+    doctrans_parser.add_argument(
+        "--filename",
+        help="Python file to convert docstrings within. Edited in place.",
+        type=str,
+        required=True,
+    )
+    doctrans_parser.add_argument(
+        "--format",
+        help="The docstring format to replace existing format with.",
+        type=str,
+        choices=tuple(filterfalse(rpartial(eq, "auto"), Style.__members__.keys())),
+        required=True,
+    )
+
+    doctrans_parser_group = doctrans_parser.add_mutually_exclusive_group(required=True)
+    doctrans_parser_group.add_argument(
+        "--type-annotations",
+        help="Inline the type, i.e., annotate PEP484 (outside docstring. Requires 3.6+)",
+        dest="type_annotations",
+        action="store_true",
+    )
+    doctrans_parser_group.add_argument(
+        "--no-type-annotations",
+        help="Ensure all types are in docstring (rather than a PEP484 type annotation)",
+        dest="type_annotations",
+        action="store_false",
+    )
+
     return parser
 
 
@@ -354,18 +396,11 @@ def main(cli_argv=None, return_args=False):
             maxlen=0,
         )
 
-        if args.input_filename is None or not path.isfile(args.input_filename):
-            _parser.error(
-                "--input-file must be an existent file. Got: {!r}".format(
-                    args.input_filename
-                )
-            )
-        elif args.output_filename is None or not path.isfile(args.output_filename):
-            _parser.error(
-                "--output-file must be an existent file. Got: {!r}".format(
-                    args.output_filename
-                )
-            )
+        for filename, arg_name in (args.input_filename, "input-file"), (
+            args.output_filename,
+            "output-file",
+        ):
+            require_file_existent(_parser, filename, name=arg_name)
         sync_properties(**args_dict)
     elif command == "gen":
         if path.isfile(args.output_filename):
@@ -398,6 +433,30 @@ def main(cli_argv=None, return_args=False):
             model_paths=args.model_paths,
             routes_paths=args.routes_paths,
         )
+    elif command == "doctrans":
+        require_file_existent(_parser, args.filename, name="filename")
+        doctrans(
+            filename=args.filename,
+            docstring_format=args.format,
+            type_annotations=args.type_annotations,
+        )
+
+
+def require_file_existent(_parser, filename, name):
+    """
+    Raise SystemExit(2) if filename is None or not found
+
+    :param _parser: The argparse parser
+    :type _parser: ```ArgumentParser```
+
+    :param filename: The filename
+    :type filename: ```Optional[str]```
+
+    :param name: Argument name
+    :type name: ```str```
+    """
+    if filename is None or not path.isfile(filename):
+        _parser.error("--{} must be an existent file. Got: {!r}".format(name, filename))
 
 
 if __name__ == "__main__":
