@@ -59,14 +59,14 @@ def run_ast_test(test_case_instance, gen_ast, gold, skip_black=False):
     gold = deepcopy(gold)
 
     # if reindent_docstring:
-    #           gen_docstring = ast.get_docstring(gen_ast)
-    #           if gen_docstring is not None:
-    #               gen_ast.body[0] = set_value(
-    #                   "\n{}".format(indent(cleandoc(gen_docstring), tab))
-    #               )
-    #           gold.body[0] = set_value(
-    #               "\n{}".format(indent(ast.get_docstring(gold, clean=True), tab))
-    #           )
+    #     gen_docstring = ast.get_docstring(gen_ast)
+    #     if gen_docstring is not None:
+    #         gen_ast.body[0] = set_value(
+    #             "\n{}".format(indent(cleandoc(gen_docstring), tab))
+    #         )
+    #     gold.body[0] = set_value(
+    #       "\n{}".format(indent(ast.get_docstring(gold, clean=True), tab))
+    #     )
 
     # from meta.asttools import print_ast
     #
@@ -75,6 +75,16 @@ def run_ast_test(test_case_instance, gen_ast, gold, skip_black=False):
     # print("#gold")
     # print_ast(gold)
 
+    _gen_ast, _gold_ast = (
+        (gen_ast.body[0], gold.body[0])
+        if isinstance(gen_ast, ast.Module) and gen_ast.body
+        else (gen_ast, gold)
+    )
+    if isinstance(_gen_ast, (ast.ClassDef, ast.AsyncFunctionDef, ast.FunctionDef)):
+        test_case_instance.assertEqual(
+            *map(partial(ast.get_docstring, clean=False), (_gen_ast, _gold_ast))
+        )
+
     test_case_instance.assertEqual(
         *map(
             identity
@@ -88,7 +98,7 @@ def run_ast_test(test_case_instance, gen_ast, gold, skip_black=False):
                     string_normalization=False,
                 ),
             ),
-            map(source_transformer.to_code, (gold, gen_ast)),
+            map(source_transformer.to_code, (gen_ast, gold)),
         )
     )
 
@@ -170,7 +180,7 @@ def run_cli_test(
 
 
 def unittest_main():
-    """ Runs unittest.main if __main__ """
+    """Runs unittest.main if __main__"""
     if __name__ == "__main__":
         main()
 
